package com.livelike.flutter.livelike_flutter_sdk

import android.content.Context
import android.os.Handler
import android.os.Looper
import android.util.Log
import androidx.annotation.NonNull
import com.livelike.engagementsdk.EngagementSDK
import com.livelike.engagementsdk.EpochTime
import com.livelike.engagementsdk.LiveLikeContentSession
import com.livelike.engagementsdk.chat.LiveLikeChatSession
import com.livelike.engagementsdk.core.AccessTokenDelegate
import com.livelike.engagementsdk.core.data.models.LeaderBoard
import com.livelike.engagementsdk.publicapis.ErrorDelegate
import com.livelike.engagementsdk.publicapis.LiveLikeCallback
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import org.json.JSONArray
import org.json.JSONException
import org.json.JSONObject


/** LivelikeFlutterSdkPlugin */
class LivelikeFlutterSdkPlugin : FlutterPlugin, MethodCallHandler {
    internal lateinit var widgetStreamChannel: EventChannel
    private lateinit var errorDelegateChannel: EventChannel
    private lateinit var accessTokenDelegateChannel: EventChannel
    internal val widgetStreamHandler = LiveLikeFlutterStreamHandler()

    /// The MethodChannel that will the communication between Flutter and native Android
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel
    lateinit var engagementSDK: EngagementSDK
    private var applicationContext: Context? = null
    val sessionMap = hashMapOf<String, LiveLikeContentSession>()
    val chatSessionMap = hashMapOf<String, LiveLikeChatSession>()
    private val uiThreadHandler: Handler = Handler(Looper.getMainLooper())
    private var currentPDTTimeMap = mutableMapOf<String, Long>()

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "livelike_flutter_sdk")
        channel.setMethodCallHandler(this)
        applicationContext = flutterPluginBinding.applicationContext
        flutterPluginBinding.platformViewRegistry
            .registerViewFactory(
                "chatView",
                ChatViewFactory(this, flutterPluginBinding.binaryMessenger)
            )
        flutterPluginBinding.platformViewRegistry
            .registerViewFactory(
                "widgetView",
                WidgetViewFactory(this, flutterPluginBinding.binaryMessenger)
            )
        accessTokenDelegateChannel =
            EventChannel(flutterPluginBinding.binaryMessenger, "flutter_sdk_access_token_delegate")
        errorDelegateChannel =
            EventChannel(flutterPluginBinding.binaryMessenger, "flutter_sdk_error_delegate")
        widgetStreamChannel = EventChannel(flutterPluginBinding.binaryMessenger, "widget_stream")
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
//        Log.d("LiveLike", "Method Call: ${call.method}")
        when (call.method) {
            "getPlatformVersion" -> {
                result.success("Android ${android.os.Build.VERSION.RELEASE}")
            }
            "initSDK" -> {
                val clientId: String? = call.argument<String>("clientId")
                clientId?.let {
                    val accessToken: String? = call.argument<String>("accessToken")
                    val usingAccessToken: Boolean? = call.argument<Boolean>("usingAccessToken")
                    val accessTokenEventHandler = LiveLikeFlutterStreamHandler()
                    accessTokenDelegateChannel.setStreamHandler(accessTokenEventHandler)
                    val errorEventHandler = LiveLikeFlutterStreamHandler()
                    errorDelegateChannel.setStreamHandler(errorEventHandler)
                    applicationContext?.let { context ->
                        engagementSDK = EngagementSDK(clientId!!,
                            context,
                            accessTokenDelegate = when (usingAccessToken) {
                                true -> object : AccessTokenDelegate {
                                    override fun getAccessToken(): String? = accessToken

                                    override fun storeAccessToken(accessToken: String?) {
                                        Log.d(
                                            "LiveLike",
                                            "Access Token:$accessToken, Check:${accessTokenEventHandler.eventSink}"
                                        )
                                        accessTokenEventHandler.eventSink?.success(accessToken)
                                    }
                                }
                                else -> null
                            },
                            errorDelegate = object : ErrorDelegate() {
                                override fun onError(error: String) {
                                    errorEventHandler.eventSink?.success(error)
                                }
                            })
                        result.success(null)
                        return
                    }
                    result.error("No Context", "Application Context is null", null)
                    return
                }
                result.error("", "Client Id cannot be empty", null)
            }
            "setVideoTimePDT" -> {
                val videoPDTTime = call.argument<Int>("videoTimePDT")
                val programId = call.argument<String>("programId")
                val chatRoomId = call.argument<String>("chatRoomId")
                programId?.let {
                    currentPDTTimeMap.put(it, videoPDTTime?.toLong() ?: 0L)
                }
                chatRoomId?.let {
                    currentPDTTimeMap.put(it, videoPDTTime?.toLong() ?: 0L)
                }
//                Log.d("LiveLike", "PDT: $videoPDTTime")
            }
            "createSession" -> {
                val programId = call.argument<String>("programId")
                val errorEventHandler = LiveLikeFlutterStreamHandler()
                errorDelegateChannel.setStreamHandler(errorEventHandler)
                programId?.let {
                    sessionMap[programId] = engagementSDK.createContentSession(
                        programId,
                        timecodeGetter = object : EngagementSDK.TimecodeGetter {
                            override fun getTimecode(): EpochTime {
//                                Log.d("LiveLike", "TimeCode")
                                return EpochTime(currentPDTTimeMap[programId] ?: 0L)
                            }
                        },
                        errorDelegate = object : ErrorDelegate() {
                            override fun onError(error: String) {
                                errorEventHandler.eventSink?.success(error)
                            }
                        }).apply {
                        chatSessionMap[this.chatSession.getCurrentChatRoom()] = this.chatSession
                        widgetStreamChannel.setStreamHandler(widgetStreamHandler)
                        result.success(
                            hashMapOf(
                                "programId" to this.contentSessionId(),
                                "chatRoomId" to this.chatSession.getCurrentChatRoom()
                            )
                        )
                    }
                    return
                }
                result.error("", "Error in creating the session", null)
            }
            "resumeSession" -> {
                val programId = call.argument<String>("programId")
                programId?.let {
                    sessionMap[programId]?.resume()
                    result.success(null)
                    return
                }
                result.error("", "Error in resuming the session", null)
            }
            "pauseSession" -> {
                val programId = call.argument<String>("programId")
                programId?.let {
                    sessionMap[programId]?.pause()
                    result.success(null)
                    return
                }
                result.error("", "Error in pausing the session", null)
            }
            "closeSession" -> {
                val programId = call.argument<String>("programId")
                programId?.let {
                    sessionMap[programId]?.close()
                    sessionMap.remove(programId)
                    result.success(null)
                    return
                }
                result.error("", "Error in closing the session", null)
            }
            "createChatSession" -> {
                val chatRoomId = call.argument<String>("chatRoomId")
                val errorEventHandler = LiveLikeFlutterStreamHandler()
                errorDelegateChannel.setStreamHandler(errorEventHandler)
                chatRoomId?.let {
                    chatSessionMap[chatRoomId] =
                        engagementSDK.createChatSession(errorDelegate = object : ErrorDelegate() {
                            override fun onError(error: String) {
                                errorEventHandler.eventSink?.success(error)
                            }
                        }, timecodeGetter = object : EngagementSDK.TimecodeGetter {
                            override fun getTimecode(): EpochTime {
                                return EpochTime(currentPDTTimeMap[chatRoomId] ?: 0L)
                            }
                        }).apply {
                            this.connectToChatRoom(
                                chatRoomId,
                                callback = object : LiveLikeCallback<Unit>() {
                                    override fun onResponse(res: Unit?, error: String?) {
                                        uiThreadHandler.post {
                                            res?.let {
                                                result.success(hashMapOf("chatRoomId" to this@apply.getCurrentChatRoom()))
                                            }
                                            error?.let {
                                                result.error("", error, null)
                                            }
                                        }
                                    }
                                })
                        }
                    return
                }
                result.error("", "Error in creating in chat session", null)
            }
            "resumeChatSession" -> {
                val chatRoomId = call.argument<String>("chatRoomId")
                chatRoomId?.let {
                    chatSessionMap[chatRoomId]?.resume()
                    result.success(null)
                    return
                }
                result.error("", "Error in resuming the chat session", null)
            }
            "pauseChatSession" -> {
                val chatRoomId = call.argument<String>("chatRoomId")
                chatRoomId?.let {
                    chatSessionMap[chatRoomId]?.pause()
                    result.success(null)
                    return
                }
                result.error("", "Error in pausing the chat session", null)
            }
            "closeChatSession" -> {
                val chatRoomId = call.argument<String>("chatRoomId")
                chatRoomId?.let {
                    chatSessionMap[chatRoomId]?.close()
                    chatSessionMap.remove(chatRoomId)
                    result.success(null)
                    return
                }
                result.error("", "Error in closing the chat session", null)
            }
            "getLeaderBoards" -> {
                val programId = call.argument<String>("programId")
                programId?.let {
                    engagementSDK.getLeaderBoardsForProgram(programId,
                        object : LiveLikeCallback<List<LeaderBoard>>() {
                            override fun onResponse(list: List<LeaderBoard>?, error: String?) {
                                list?.let {
                                    result.success(list)
                                }
                            }
                        })
                    return
                }
                result.error("", "Program Id cannot be null", null)
            }
            else -> {
                result.notImplemented()
            }
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }
}

class LiveLikeFlutterStreamHandler : EventChannel.StreamHandler {
    private var _eventSink: EventChannel.EventSink? = null

    val eventSink: EventChannel.EventSink?
        get() = _eventSink

    override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
        Log.d("LiveLike", "onListen: $events")
        this._eventSink = events
    }

    override fun onCancel(arguments: Any?) {
        this._eventSink = null
    }

}

@Throws(JSONException::class)
fun toMap(`object`: JSONObject): Map<String, Any> {
    val map: MutableMap<String, Any> = HashMap()
    val keysItr = `object`.keys()
    while (keysItr.hasNext()) {
        val key = keysItr.next()
        var value = `object`[key]
        if (value is JSONArray) {
            value = toList(value)
        } else if (value is JSONObject) {
            value = toMap(value)
        }
        map[key] = value
    }
    return map
}

@Throws(JSONException::class)
fun toList(array: JSONArray): List<Any> {
    val list: MutableList<Any> = ArrayList()
    for (i in 0 until array.length()) {
        var value = array[i]
        if (value is JSONArray) {
            value = toList(value)
        } else if (value is JSONObject) {
            value = toMap(value)
        }
        list.add(value)
    }
    return list
}